# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2
from driverng.splitted import Splitted, String


class BrukerCsvBase(Splitted):
    SEPARATOR = ";"


class BrukerCsvOrder(BrukerCsvBase):
    FIELDS = (
        String("target_plate_position"),
        String("sample_id"),
        String("patient_name"),
    )


class BrukerCsvMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.ORDER: BrukerCsvOrder}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "bruker_csv", "label": "BRUKER CSV"}

    MESSAGE_TYPE_MAP = {MessageV2.ORDER: BrukerCsvOrder}

    def create(self):
        return BrukerCsvMultiple()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = BrukerCsvMultiple()
        obj.parse(data)
        return obj

    def encode_worklist(self, worklist, context):
        message = self.create()
        for sample in worklist.get_plan_de_plaque_worklist():
            if sample.bruker_type_echantillon == "1":
                tube_bruker = "BTS"
            else:
                tube_bruker = "{}-{}".format(sample.bruker_position, sample.sample_id)
            if sample.last_name and sample.first_name:
                nom_patient = "{} {}".format(sample.last_name, sample.first_name)
            else:
                nom_patient = ""
            message.add(
                MessageV2.ORDER,
                target_plate_position=sample.bruker_position,
                sample_id=tube_bruker,
                patient_name=nom_patient,
            )
        return message
