# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import xml.etree.cElementTree as ET

from driverng import MessageV2, utils, Worklist


class Driver(MessageV2):
    SPEC = {"code": "filmarray", "label": "Filmarray Biomerieux"}
    ENCODING = "utf8"

    def parse(self, data):
        return ET.fromstring(data.encode(self.ENCODING))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, "utf-8"))

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for request in message.findall("requestResult"):
            for test_order in request.findall("testOrder"):
                sample_id = test_order.find("specimen").find("specimenIdentifier").text
                sample = worklist.get_or_create(sample_id=sample_id, type="patient")
                for test in test_order.findall("test"):
                    resultats = []
                    for result in test.findall("resultGroup"):
                        for res in result.findall("result"):
                            # code test du résultat
                            resultid = res.find("resultID")
                            testid = resultid.find("resultTestCode").text

                            # résultat
                            value = res.find("value")
                            testresult = value.find("testResult")
                            resu = testresult.find("observationValue").text
                            date = datetime.datetime.strptime(
                                res.find("resultDateTime").text, "%Y%m%d%H%M%S"
                            )
                            resultats.append((testid, resu, date))
                    for code, resu, date in resultats:
                        result = sample.results.add(code=code, value=resu, date=date)
                        for disposableData in test.findall("disposableData"):
                            for disposable in disposableData.findall("disposable"):
                                lot = disposable.findtext("lotNumber")
                                if lot:
                                    result.lots.add(type="reagent", lot=lot)
        return worklist
