# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.tests import BaseTest


class TestAdvia(BaseTest):
    MAPPING_CLASS = "advia"

    def test_order(self):
        self.assertOrderMessageEqual(
            self.WORKLIST_NEW_ADD_DEL,
            (
                "Y  U  123456                                 L036666666       NYME ANO                        5/13/2011 F 06/03/13 1103 SER    PRS    \n"
                "00A            00B            "
            ),
        )

    def test_query(self):
        self.assertQueryEqual("Q 00470112185945", ["470112185945"])

    def test_void_query(self):
        self.assertVoidOrderEqual("Q 00470112185945", "N W 00470112185945")

    @staticmethod
    def unindent(data):
        return data

    def test_result(self):
        date = datetime.datetime(2013, 12, 18, 15, 0, 47)
        self.assertResultEqual(
            (
                "R 470112185945   040-01           12/18/13 15:00:47   \n"
                " 5213.57  GF   113.57  GF  28*****  GF   2 5.85* GF |CC|  3 17.1* GF |CC|  5 76.9* GF |CC|  4 44.9* GF |CC|  6 29.2* GF |CC|  7 38.0* GF |CC| 51 41.4* GF |CC| 10  157  GF  11  9.9  GF  20 13.3  GF  23    0  GF  24  0.6* GF |BC| 21  0.6* GF |BC| 22 78.6  GF  25  7.6* GF |BC| 14 1.80  GF  17    0  GF  18 0.09* GF |BC| 15 0.08* GF |BC| 1610.66  GF  19 1.03* GF |BC| 5014.57  GF 650  0.4  GF 908  0.2  GF  31  0.3  GF 300 17.5  GF   8 14.4* GF |CC|  9 2.12* GF |CC| 27 -3.1  GF  26 2.29  GF 999 18.6  GF  37    1  GF  41    3  GF  75    3  GF  44    2  GF  43    2  GF 19160.71  GF 19267.35  GF 311 12.0  GF 312  6.0  GF  29*****  GF  66 17.3  GF 651  0.3  GF  12 39.8  GF 250  370  GF "
            ),
            {
                "470112185945": {
                    "rack": "040-01",
                    "resultats": [
                        {"date": date, "resu": "13.57", "test": "52"},
                        {"date": date, "resu": "13.57", "test": "1"},
                        {"date": date, "resu": "*****", "test": "28"},
                        {"alarmes": ["CC"], "date": date, "resu": "5.85", "test": "2"},
                        {"alarmes": ["CC"], "date": date, "resu": "17.1", "test": "3"},
                        {"alarmes": ["CC"], "date": date, "resu": "76.9", "test": "5"},
                        {"alarmes": ["CC"], "date": date, "resu": "44.9", "test": "4"},
                        {"alarmes": ["CC"], "date": date, "resu": "29.2", "test": "6"},
                        {"alarmes": ["CC"], "date": date, "resu": "38.0", "test": "7"},
                        {"alarmes": ["CC"], "date": date, "resu": "41.4", "test": "51"},
                        {"date": date, "resu": "157", "test": "10"},
                        {"date": date, "resu": "9.9", "test": "11"},
                        {"date": date, "resu": "13.3", "test": "20"},
                        {"date": date, "resu": "0", "test": "23"},
                        {"alarmes": ["BC"], "date": date, "resu": "0.6", "test": "24"},
                        {"alarmes": ["BC"], "date": date, "resu": "0.6", "test": "21"},
                        {"date": date, "resu": "78.6", "test": "22"},
                        {"alarmes": ["BC"], "date": date, "resu": "7.6", "test": "25"},
                        {"date": date, "resu": "1.80", "test": "14"},
                        {"date": date, "resu": "0", "test": "17"},
                        {"alarmes": ["BC"], "date": date, "resu": "0.09", "test": "18"},
                        {"alarmes": ["BC"], "date": date, "resu": "0.08", "test": "15"},
                        {"date": date, "resu": "10.66", "test": "16"},
                        {"alarmes": ["BC"], "date": date, "resu": "1.03", "test": "19"},
                        {"date": date, "resu": "14.57", "test": "50"},
                        {"date": date, "resu": "0.4", "test": "650"},
                        {"date": date, "resu": "0.2", "test": "908"},
                        {"date": date, "resu": "0.3", "test": "31"},
                        {"date": date, "resu": "17.5", "test": "300"},
                        {"alarmes": ["CC"], "date": date, "resu": "14.4", "test": "8"},
                        {"alarmes": ["CC"], "date": date, "resu": "2.12", "test": "9"},
                        {"date": date, "resu": "-3.1", "test": "27"},
                        {"date": date, "resu": "2.29", "test": "26"},
                        {"date": date, "resu": "18.6", "test": "999"},
                        {"date": date, "resu": "1", "test": "37"},
                        {"date": date, "resu": "3", "test": "41"},
                        {"date": date, "resu": "3", "test": "75"},
                        {"date": date, "resu": "2", "test": "44"},
                        {"date": date, "resu": "2", "test": "43"},
                        {"date": date, "resu": "60.71", "test": "191"},
                        {"date": date, "resu": "67.35", "test": "192"},
                        {"date": date, "resu": "12.0", "test": "311"},
                        {"date": date, "resu": "6.0", "test": "312"},
                        {"date": date, "resu": "*****", "test": "29"},
                        {"date": date, "resu": "17.3", "test": "66"},
                        {"date": date, "resu": "0.3", "test": "651"},
                        {"date": date, "resu": "39.8", "test": "12"},
                        {"date": date, "resu": "370", "test": "250"},
                        {"date": date, "resu": "BC;CC", "test": "ALARM"},
                    ],
                }
            },
        )

    def test_qc(self):
        date = datetime.datetime(2013, 12, 18, 15, 0, 48)
        self.assertQCEqual(
            (
                "R 9000000                         12/18/13 15:00:48   \n"
                " 52 6.91  GF   1 6.91  GF   2 4.73  GF   3 13.8  GF   5 90.2  GF   4 42.2  GF   6 29.4  GF   7 32.5  GF  51 33.2  GF  10  251  GF  11  9.5  GF   8 14.1  GF   9 2.58  GF 311 14.1  GF 312 14.5  GF "
            ),
            [
                {"date": date, "lot": "9000000", "resu": "6.91", "test": "52"},
                {"date": date, "lot": "9000000", "resu": "6.91", "test": "1"},
                {"date": date, "lot": "9000000", "resu": "4.73", "test": "2"},
                {"date": date, "lot": "9000000", "resu": "13.8", "test": "3"},
                {"date": date, "lot": "9000000", "resu": "90.2", "test": "5"},
                {"date": date, "lot": "9000000", "resu": "42.2", "test": "4"},
                {"date": date, "lot": "9000000", "resu": "29.4", "test": "6"},
                {"date": date, "lot": "9000000", "resu": "32.5", "test": "7"},
                {"date": date, "lot": "9000000", "resu": "33.2", "test": "51"},
                {"date": date, "lot": "9000000", "resu": "251", "test": "10"},
                {"date": date, "lot": "9000000", "resu": "9.5", "test": "11"},
                {"date": date, "lot": "9000000", "resu": "14.1", "test": "8"},
                {"date": date, "lot": "9000000", "resu": "2.58", "test": "9"},
                {"date": date, "lot": "9000000", "resu": "14.1", "test": "311"},
                {"date": date, "lot": "9000000", "resu": "14.5", "test": "312"},
            ],
        )

    def test_qc_ignore(self):
        date = datetime.datetime(2015, 9, 10, 10, 47, 24)
        self.assertQCEqual(
            (
                "R 9207500        008-01           09/10/15 10:47:24   \n"
                "  1 7.05*Dcs |WC| 50 8.10 Dcs   2 4.25 Dcs   3 11.4 Dcs   4 33.5 Dcs   5 78.7 Dcs"
            ),
            [
                {
                    "date": date,
                    "lot": "9207500",
                    "status": "D",
                    "resu": "7.05",
                    "test": "1",
                },
                {
                    "date": date,
                    "lot": "9207500",
                    "status": "D",
                    "resu": "8.10",
                    "test": "50",
                },
                {
                    "date": date,
                    "lot": "9207500",
                    "status": "D",
                    "resu": "4.25",
                    "test": "2",
                },
                {
                    "date": date,
                    "lot": "9207500",
                    "status": "D",
                    "resu": "11.4",
                    "test": "3",
                },
                {
                    "date": date,
                    "lot": "9207500",
                    "status": "D",
                    "resu": "33.5",
                    "test": "4",
                },
                {
                    "date": date,
                    "lot": "9207500",
                    "status": "D",
                    "resu": "78.7",
                    "test": "5",
                },
            ],
        )
