# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "atellica"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].orders[0].dilution = "10"
    message = self.order(
        worklist=worklist,
        context={"control_id": "1521651964637", "awos_id": "1521651964639"},
    )
    expected = """
    MSH|^~\\&|LIS_ID|""|UIW_LIS|""|20130715141912||OML^O33^OML_O33|1521651964637|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28^IHE
    PID|||036666666||NYME^ANO^^^^^L||20110513|F
    PV1||I|LOC
    SPM|1|123456||Serum^^|||||||P^^||||||20130513114242|20130603110342
    SAC|||123456
    ORC|NW
    TQ1|||||||||A^^
    OBR||1521651964639||A^^|||||||G
    TCD|A^^||10
    SPM|1|123456||Serum^^|||||||P^^||||||20130513114242|20130603110342
    SAC|||123456
    ORC|NW
    TQ1|||||||||A^^
    OBR||1521651964639||B^^|||||||G
    TCD|B^^
    """
    assert message == expected


def test_decode_result_production(self):
    date = datetime.datetime(2018, 2, 21, 11, 23, 55)
    worklist = self.decode(
        """
    MSH|^~\\&|UIW_LIS|""|LIS_ID|""|20180221140658+0100||OUL^R22^OUL_R22|7bc2ed8839c743b4b2c9188ac2141454|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE~LAW_REFLEX^IHE~LAW_SPECIMEN^IHE~LAW_RERUN^IHE~LAW_CONTAINER^IHE~LAW_DILUTIONS^IHE~LAW_PAT_DEM^IHE
    PID|1||1758463^^^^PT||ESSAI ATELLICA^MULTQ 1^^^^^L||19501129000000|M
    SPM|1|999000000093^""|""^""|SER^Serum^HL70487|||||||P^Patient specimen^HL70369
    SAC|||999000000093
    ORC|OK||||CM
    OBR|1|""||Ca^Ca^99SiemensHDXTestCode
    TQ1|||||||||R^Routine^HL70486
    OBX|1|NM|Ca.RLU^Ca^99SiemensHDXTestCode|1^1|436.8913|mg/L^mg/L^UCUM||""^""^HL70078|||F|||||LabManager^No Review||NextGenAnalyzer^SiemensHDX1~SP00135^SystemSN~CM00140^ModuleSN|20180221112355||||||||||RSLT
TCD|Ca^Ca^99SiemensHDXTestCode
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Not applicable^HL70383|SR^Single test reagent^HL70384|||||||||||||270357
    NTE|1|Z|Lot=128433;DateTime=2/21/2018 11:14:46 AM;Status=Valid;ID=290;|CALINFO^^99SiemensHDX
    NTE|2|Z|(Name;Level;Lot;Result;DateTime)=(MTQ 1;1;45751;52.8;20180216102246)(MTQ 2;2;45772;100.6;20180216102255)(MTQ 3;3;45773;134.4;20180216102416)|QCINFO^^99SiemensHDX
    OBX|2|NM|Ca.DOSE^Ca^99SiemensHDXTestCode|1^1|85.1|mg/L^mg/L^UCUM||""^""^HL70078|||F|||||LabManager^No Review||NextGenAnalyzer^SiemensHDX~SP00135^SystemSN~CM00140^ModuleSN|20180221112355||103103||||||||RSLT
    TCD|Ca^Ca^99SiemensHDXTestCode
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Not applicable^HL70383|SR^Single test reagent^HL70384|||||||||||||270357
    NTE|1|Z|Lot=128433;DateTime=2/21/2018 11:14:46 AM;Status=Valid;ID=290;|CALINFO^^99SiemensHDX
    NTE|2|Z|(Name;Level;Lot;Result;DateTime)=(MTQ 1;1;45751;52.8;20180216102246)(MTQ 2;2;45772;100.6;20180216102255)(MTQ 3;3;45773;134.4;20180216102416)|QCINFO^^99SiemensHDX
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Ca.RLU",
                        "date": date,
                        "dilution": "",
                        "flags": [],
                        "comments": [],
                        "instrument": "CM00140",
                        "operator": "LabManager",
                        "unit": "mg/L",
                        "value": "436.8913",
                        "lots": [{"lot": "270357", "type": "reagent"}],
                    },
                    {
                        "code": "Ca",
                        "date": date,
                        "dilution": "",
                        "flags": [],
                        "comments": [],
                        "instrument": "CM00140",
                        "operator": "LabManager",
                        "unit": "mg/L",
                        "value": "436.8913",
                        "lots": [{"lot": "270357", "type": "reagent"}],
                    },
                    {
                        "code": "Ca.DOSE",
                        "date": date,
                        "dilution": "",
                        "flags": [],
                        "comments": [],
                        "instrument": "CM00140",
                        "operator": "LabManager",
                        "unit": "mg/L",
                        "value": "85.1",
                        "lots": [{"lot": "270357", "type": "reagent"}],
                    },
                    {
                        "code": "Ca",
                        "date": date,
                        "dilution": "",
                        "flags": [],
                        "comments": [],
                        "instrument": "CM00140",
                        "operator": "LabManager",
                        "unit": "mg/L",
                        "value": "85.1",
                        "lots": [{"lot": "270357", "type": "reagent"}],
                    },
                ],
                "sample_id": "999000000093",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result2(self):
    worklist = self.decode(
        """
    MSH|^~\\&|UIW_LIS|""|LIS_ID|""|20180411120814+0200||OUL^R22^OUL_R22|a233ee27b6154f29bacfd69ecba19162|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE~LAW_REFLEX^IHE~LAW_SPECIMEN^IHE~LAW_RERUN^IHE~LAW_CONTAINER^IHE~LAW_DILUTIONS^IHE~LAW_PAT_DEM^IHE
    PID|1||1000000012^^^^PT||ESSAI^TEST^^^^^L|||U
    PV1|1|I
    SPM|1|903130000018^""|""^""|SER^Serum^HL70487|||||||P^Patient specimen^HL70369
    SAC|||903130000018
    OBR|1|1523441259733||TP^TP^99SiemensHDXTestCode
    ORC|OK||||CM
    TQ1|||||||||R^Routine^HL70486
    OBX|1|ST|TP^TP^99SiemensHDXTestCode|1|""|||""^""^HL70078|||X|||||""||NextGenAnalyzer^SiemensHDX~SP00148^SystemSN|20180411120814||||||||||RSLT
    TCD|TP^TP^99SiemensHDXTestCode
    NTE|1|Z|Insufficient Sample|FLAG^^99SiemensHDXResultFlag
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "TP",
                        "flags": ["Insufficient Sample"],
                        "date": datetime.datetime(2018, 4, 11, 12, 8, 14),
                        "dilution": "",
                        "comments": [],
                        "instrument": "SP00148",
                        "operator": '""',
                        "unit": "",
                        "value": '""',
                    },
                    {
                        "code": "TP.DOSE",
                        "flags": ["Insufficient Sample"],
                        "date": datetime.datetime(2018, 4, 11, 12, 8, 14),
                        "dilution": "",
                        "comments": [],
                        "instrument": "SP00148",
                        "operator": '""',
                        "unit": "",
                        "value": '""',
                    },
                    {
                        "code": "TP.RLU",
                        "flags": ["Insufficient Sample"],
                        "date": datetime.datetime(2018, 4, 11, 12, 8, 14),
                        "dilution": "",
                        "comments": [],
                        "instrument": "SP00148",
                        "operator": '""',
                        "unit": "",
                        "value": '""',
                    },
                    {
                        "code": "TP.INDX",
                        "flags": ["Insufficient Sample"],
                        "date": datetime.datetime(2018, 4, 11, 12, 8, 14),
                        "dilution": "",
                        "comments": [],
                        "instrument": "SP00148",
                        "operator": '""',
                        "unit": "",
                        "value": '""',
                    },
                    {
                        "code": "TP.INTR",
                        "flags": ["Insufficient Sample"],
                        "date": datetime.datetime(2018, 4, 11, 12, 8, 14),
                        "dilution": "",
                        "comments": [],
                        "instrument": "SP00148",
                        "operator": '""',
                        "unit": "",
                        "value": '""',
                    },
                    {
                        "code": "TP.COFF",
                        "flags": ["Insufficient Sample"],
                        "date": datetime.datetime(2018, 4, 11, 12, 8, 14),
                        "dilution": "",
                        "comments": [],
                        "instrument": "SP00148",
                        "operator": '""',
                        "unit": "",
                        "value": '""',
                    },
                ],
                "sample_id": "903130000018",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2018, 1, 18, 9, 46, 10)
    worklist = self.decode(
        """
    MSH|^~\\&|Siemens Analyzer|SiemensLab|xyzco|AM_xyzco_xyzlab|20150706094746-0400||OUL^R22^OUL_R22|61a8d7e52e4f426dbb295d7c35b22cab|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    PID|1||PIDXYZ213^^^^PT||PROVA3^PROVA3^^^^^L||19880110000000|F
    PV1|1|U|^103
    SPM|1|31000213^""|""^""|SER^Serum^HL70487|||||||P^Patient specimen^HL70369
    SAC|||31000213||||||||||003969|A1
    ORC|OK||||CM
    OBR|1|38fcd339ba01a7308806de6c323d4928||2951-2^Na-S^LN
    OBX|1|NM|2951-2.RLU^Na-S^LN|1^1^^1017|53355|U/mL^U/mL^UCUM||""^""^HL70078|||F|||||siemensinternal||SiemensAnalyzer^SiemensHDX~5-5-5-5-5-5-5^SystemSN~4-4-4-4-4-4-4^ModuleSN|20180118094610||||||||||RSLT
    TCD|2951-2^Na-S^LN
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Notapplicable^HL70383||SR^Single testreagent^HL70384||||||||||||103
    NTE|1|Z|Lot=C015;DateTime=20150629113137;Status=Due;
    NTE|2|Z|(Name;Level;Lot;Result;DateTime)=(QELF;1;4444;32.77;20150629144912)
    OBX|2|NM|2951-2.COFF^Na-S^LN|1^1^^1017|1.0|U/mL^U/mL^UCUM||""^""^HL70078|||F|||||siemensinternal||SiemensAnalyzer^SiemensHDX~5-5-5-5-5-5-5^SystemSN~4-4-4-4-4-4-4^ModuleSN|20180118094610||||||||||RSLT
    TCD|2951-2^Na-S^LN
    OBX|3|NM|2951-2.DOSE^Na-S^LN|1^1^^1017|24.05|U/mL^U/mL^UCUM||""^""^HL70078|||F|||||siemensinternal^NoReview||SiemensAnalyzer^SiemensHDX~5-5-5-5-5-5-5^SystemSN~4-4-4-4-4-4-4^ModuleSN|20180118094610||||||||||RSLT
    TCD|2951-2^Na-S^LN
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Notapplicable^HL70383||SR^Single testreagent^HL70384||||||||||||103
    NTE|1|Z|Lot=C015;DateTime=20150629113137;Status=Due;
    NTE|2|Z|(Name;Level;Lot;Result;DateTime)=(QELF;1;4444;32.77;20150629144912)
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "2951-2.RLU",
                        "value": "53355",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": date,
                        "dilution": "",
                        "unit": "U/mL",
                        "lots": [{"lot": "103", "type": "reagent"}],
                    },
                    {
                        "code": "2951-2",
                        "value": "53355",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": date,
                        "dilution": "",
                        "unit": "U/mL",
                        "lots": [{"lot": "103", "type": "reagent"}],
                    },
                    {
                        "code": "2951-2.COFF",
                        "value": "1.0",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": date,
                        "dilution": "",
                        "unit": "U/mL",
                    },
                    {
                        "code": "2951-2",
                        "value": "1.0",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": date,
                        "dilution": "",
                        "unit": "U/mL",
                    },
                    {
                        "code": "2951-2.DOSE",
                        "value": "24.05",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": date,
                        "dilution": "",
                        "unit": "U/mL",
                        "lots": [{"lot": "103", "type": "reagent"}],
                    },
                    {
                        "code": "2951-2",
                        "value": "24.05",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": date,
                        "dilution": "",
                        "unit": "U/mL",
                        "lots": [{"lot": "103", "type": "reagent"}],
                    },
                ],
                "rack": "003969-A1",
                "sample_id": "31000213",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Siemens Analyzer|SiemensLab|xyzco|AM_xyzco_xyzlab|20150706094746-0400||OUL^R22^OUL_R22|61a8d7e52e4f426dbb295d7c35b22cab|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    PID|1||PIDXYZ213^^^^PT||PROVA3^PROVA3^^^^^L||19880110000000|F
    PV1|1|U|^103
    SPM|1|31000213^""|""^""|SER^Serum^HL70487|||||||Q^Control specimen^HL70369
    SAC|||31000213
    OBR|1|38fcd339ba01a7308806de6c323d4928||2951-2^Na-S^LN
    OBX|1|NM|2951-2.RLU^Na-S^LN|1^1^^1017|53355|U/mL^U/mL^UCUM||""^""^HL70078|||F|||||siemensinternal||SiemensAnalyzer^SiemensHDX~5-5-5-5-5-5-5^SystemSN~4-4-4-4-4-4-4^ModuleSN|20180118094610||||||||||RSLT
    INV|ControlName1^Control^99SiemensHDX|NA^Not applicable^HL70383|CO^Control^HL70384|||||||||||||4444
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "2951-2.RLU",
                        "value": "53355",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": datetime.datetime(2018, 1, 18, 9, 46, 10),
                        "dilution": "",
                        "unit": "U/mL",
                    },
                    {
                        "code": "2951-2",
                        "value": "53355",
                        "flags": [],
                        "comments": [],
                        "instrument": "4-4-4-4-4-4-4",
                        "operator": "siemensinternal",
                        "date": datetime.datetime(2018, 1, 18, 9, 46, 10),
                        "dilution": "",
                        "unit": "U/mL",
                    },
                ],
                "sample_id": "31000213",
                "control_lot": "4444",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_control_in_errors(self):
    worklist = self.decode(
        """
    MSH|^~\\&|UIW_LIS|""|LIS_ID|""|20180328100836+0200||OUL^R22^OUL_R22|4b19ac9e52cc4c0eb01ee7b9eb463476|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE~LAW_REFLEX^IHE~LAW_SPECIMEN^IHE~LAW_RERUN^IHE~LAW_CONTAINER^IHE~LAW_DILUTIONS^IHE~LAW_PAT_DEM^IHE
    SPM|1|1234^""|""^""|SER^Serum^HL70487|||||||Q^Control specimen^HL70369
    SAC|||1234
    INV|FT4 QC Test^Control^99SiemensHDX|NA^Not applicable^HL70383|CO^Control^HL70384|||||||||||||1234
    OBR|1|""||TP^TP^99SiemensHDXTestCode
    ORC|OK||||CM
    TQ1|||||||||R^Routine^HL70486
    OBX|1|ST|TP^TP^99SiemensHDXTestCode|1|""|||""^""^HL70078|||X|||||""||NextGenAnalyzer^SiemensHDX~SP00148^SystemSN|20180328100836||||||||||RSLT
    TCD|TP^TP^99SiemensHDXTestCode
    NTE|1|Z|Module Unavailable|FLAG^^99SiemensHDXResultFlag
    NTE|2|Z|Missing Module Capability|FLAG^^99SiemensHDXResultFlag
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "TP",
                        "value": '""',
                        "unit": "",
                        "instrument": "SP00148",
                        "operator": '""',
                        "date": datetime.datetime(2018, 3, 28, 10, 8, 36),
                    }
                ],
                "sample_id": "1234",
                "control_lot": "1234",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_control_multiple(self):
    worklist = self.decode(
        """
    MSH|^~\\&|UIW_LIS|""|LIS_ID|""|20181019085137+0200||OUL^R22^OUL_R22|fc3333e690ea419e99bdb471e52617d9|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE~LAW_REFLEX^IHE~LAW_SPECIMEN^IHE~LAW_RERUN^IHE~LAW_CONTAINER^IHE~LAW_DILUTIONS^IHE~LAW_PAT_DEM^IHE
    SPM|1|45801^""|""^""|SER^Serum^HL70487|||||||Q^Control specimen^HL70369
    SAC|||45801
    INV|Multiqual 1^Control^99SiemensHDX|NA^Not applicable^HL70383|CO^Control^HL70384|||||||||||||45801
    OBR|1|""||NA^NA^99SiemensHDXTestCode
    ORC|OK||||CM
    TQ1|||||||||R^Routine^HL70486
    OBX|1|NM|NA.DOSE^NA^99SiemensHDXTestCode|1^1|117.1|mmol/L^mmol/L^UCUM|(0 - 10000)|""^""^HL70078|||F|||||No Review~CSE||NextGenAnalyzer^SiemensHDX~SP00441^SystemSN~CM00452^ModuleSN|20181019073239||7419||||||||RSLT
    TCD|NA^NA^99SiemensHDXTestCode
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Not applicable^HL70383|SR^Single test reagent^HL70384|||||||||||||0
    NTE|1|Z|ID=107225;|CALINFO^^99SiemensHDX
    OBR|2|""||K^K^99SiemensHDXTestCode
    ORC|OK||||CM
    TQ1|||||||||R^Routine^HL70486
    OBX|1|NM|K.DOSE^K^99SiemensHDXTestCode|1^1|2.49|mmol/L^mmol/L^UCUM|(0 - 10000)|""^""^HL70078|||F|||||No Review~CSE||NextGenAnalyzer^SiemensHDX~SP00441^SystemSN~CM00452^ModuleSN|20181019073239||7420||||||||RSLT
    TCD|K^K^99SiemensHDXTestCode
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Not applicable^HL70383|SR^Single test reagent^HL70384|||||||||||||0
    NTE|1|Z|ID=107225;|CALINFO^^99SiemensHDX
    OBR|3|""||CL^CL^99SiemensHDXTestCode
    ORC|OK||||CM
    TQ1|||||||||R^Routine^HL70486
    OBX|1|NM|CL.DOSE^CL^99SiemensHDXTestCode|1^1|75.3|mmol/L^mmol/L^UCUM|(0 - 10000)|""^""^HL70078|||F|||||No Review~CSE||NextGenAnalyzer^SiemensHDX~SP00441^SystemSN~CM00452^ModuleSN|20181019073239||7421||||||||RSLT
    TCD|CL^CL^99SiemensHDXTestCode
    INV|PREAG^Single test reagent^99SiemensHDX|NA^Not applicable^HL70383|SR^Single test reagent^HL70384|||||||||||||0
    NTE|1|Z|ID=107225;|CALINFO^^99SiemensHDX
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "NA.DOSE",
                        "value": "117.1",
                        "unit": "mmol/L",
                        "instrument": "CM00452",
                        "operator": "No Review",
                        "date": datetime.datetime(2018, 10, 19, 7, 32, 39),
                    },
                    {
                        "code": "K.DOSE",
                        "value": "2.49",
                        "unit": "mmol/L",
                        "instrument": "CM00452",
                        "operator": "No Review",
                        "date": datetime.datetime(2018, 10, 19, 7, 32, 39),
                    },
                    {
                        "code": "CL.DOSE",
                        "value": "75.3",
                        "unit": "mmol/L",
                        "instrument": "CM00452",
                        "operator": "No Review",
                        "date": datetime.datetime(2018, 10, 19, 7, 32, 39),
                    },
                ],
                "sample_id": "45801",
                "control_lot": "45801",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Siemens Analyzer|SiemensLab|xyzco|AM_xyzco_xyzlab|20150706094746-0400||OUL^R22^OUL_R22|61a8d7e52e4f426dbb295d7c35b22cab|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    QPD|WOS^Work Order Step^IHELAW|d8fa0b38c26c4f91a606cd636a61246c|31000222|
    RCP|I||R^Real Time^HL70394
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "31000222"}],
    }
    assert worklist == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|Siemens Analyzer|SiemensLab|xyzco|AM_xyzco_xyzlab|20150706094746-0400||OUL^R22^OUL_R22|61a8d7e52e4f426dbb295d7c35b22cab|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
            QPD|WOS^Work Order Step^IHELAW|d8fa0b38c26c4f91a606cd636a61246c|31000222|
            RCP|I||R^Real Time^HL70394
            """
            ),
            "control_id": "20130715141912",
        },
    )
    expected = """
    MSH|^~\\&|LIS_ID|""|UIW_LIS|""|20130715141912||OML^O33^OML_O33|20130715141912|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28^IHE
    PID|||||^^^^^^L
    SPM|1|31000222||^^|||||||^^
    SAC|||31000222
    ORC|DC"""
    assert message == expected
