# -*- coding: utf8 -*-
from __future__ import unicode_literals

import driverng

parser_class = "atras"


def test_decode_query(self):
    worklist = self.decode(
        """
        H|\\^&|||^TandO^01^A01|||||HOST||P
    Q|1|^AP2K1A03WM^A2^00||ALL||||||||O
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "AP2K1A03WM", "type": "patient"}],
    }
    assert worklist == expected


def test_order_from_query(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "AR2N1A0293",
                "type": "patient",
                "orders": [
                    {"code": "04", "zone_groupement_test": "ZONE1", "action": "add"}
                ],
                "priority": "R",
            }
        ],
    )
    message = self.order(
        worklist=worklist,
        context={
            "query": self.parse(
                """
            H|\\^&|||^TandO^01^A01|||||HOST||P
            Q|1|^AR2N1A0293^A2^3||ALL||||||||O
            L|1|N
            """
            )
        },
    )
    expected = """
        H|^~\\&|||^MCA^01^A01|||||TandO||P
        O|1||AR2N1A0293^A2^3^N|^^^ZONE1|R
        L|1|N
    """
    assert message == expected


def test_encode_void_order(self):
    message = self.encode(
        self.void_worklist,
        context={
            "query": self.parse(
                """
            H|\\^&|||^TandO^01^A01|||||HOST||P
            Q|1|^AR2N1A0293^A2^3||ALL||||||||O
            L|1|N
            """
            )
        },
    )
    expected = """
        H|^~\\&|||^MCA^01^A01|||||TandO||P
        O|1||AR2N1A0293^A2^3^N|^^^99|R
        L|1|N
    """
    assert message == expected


def test_decode_results(self):
    worklist = self.decode(
        """
        H|\\^&|||^TandO^01^A01|||||HOST||P
        R|1||0505100KR9^A1^12^N^6.02^Y^149|||||F
        L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "02",
                "results": [
                    {"code": "Bin", "value": "6"},
                    {"code": "Cap Color", "value": "12"},
                    {"code": "Rack", "value": "02"},
                    {"code": "Sorting Success", "value": "Y"},
                    {"code": "Target ID", "value": "149"},
                ],
                "sample_id": "0505100KR9",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
