# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import driverng

parser_class = "cfx_biorad"


def test_decode_result(self):
    # Les lignes peuvent avoir plus de 6 champs
    worklist = self.decode(
        """
        File Name,20201209 RUN1.pcrd,,,,,
    Created By User,admin,,,,,
    Notes,,,,,,
    ID,,,,,,
    Run Started,12/09/2020 11:38:26 UTC,,,,,
    Run Ended,12/09/2020 13:15:47 UTC,,,,,
    Sample Vol,30,,,,,
    Lid Temp,105,,,,,
    Protocol File Name,SARS CoV2 ELITeMGB.prcl,,,,,
    Plate Setup File Name,20201209 RUN1.pltd,,,,,
    Base Serial Number,CT052169,,,,,
    Optical Head Serial Number,785BR27808,,,,,
    CFX Maestro Version,5.0.021.0616. ,,,,,
    ,,,,,,
    Well group,All Wells,,,,,
    Amplification step,4,,,,,
    Melt step,,,,,,
    ,,,,,,
    ,,,,,,
    Well,Fluor,Target,Content,Sample,Cq,Starting Quantity (SQ)
    A01,FAM,ORF 8,Unkn-01,5812090070,NaN,NaN
    A02,FAM,ORF 8,Unkn-09,5812090091,NaN,NaN
    A03,FAM,ORF 8,Unkn-17,5812090101,23.5126419411208,NaN
    A04,FAM,ORF 8,Unkn-25,5812090116,NaN,NaN
    A05,FAM,ORF 8,Unkn-33,,NaN,NaN
    A06,FAM,ORF 8,Unkn-41,,NaN,NaN
    """
    )
    assert worklist.samples
    trames = """
    File Name,CFX Sansure.pcrd,,,,
    Created By User,adminavec des accent èà ^ðŋħĸ,,,,
    Notes,Plaque1,,,,
    ID,,,,,
    Run Started,04/06/2020 12:37:05 UTC,,,,
    Run Ended,04/06/2020 13:00:40 UTC,,,,
    Sample Vol,50,,,,
    Lid Temp,105,,,,
    Protocol File Name,SansureProtocole.prcl,,,,
    Plate Setup File Name,CFX Sansure.pltd,,,,
    Base Serial Number,SIM01944,,,,
    Optical Head Serial Number,788BR02720,,,,
    CFX Maestro Version,4.1.2433.1219. ,,,,
    ,,,,,
    Well group,All Wells,,,,
    Amplification step,4,,,,
    Melt step,,,,,
    ,,,,,
    ,,,,,
    Well,Fluor,Target,Content,Sample,Cq
    A01,Cy5,IPC,Pos Ctrl-01,123456,14.76039106
    A02,Cy5,IPC,Unkn-07,135790,12.38063612
    A03,Cy5,IPC,Unkn-15,987652,12.37989111
    A04,Cy5,IPC,Unkn-23,123453,12.81727096
    """
    self.configure(analyseur_traites="SIM01944")
    worklist = self.decode(trames)
    date_resu = datetime.datetime(2020, 4, 6, 13, 0, 40)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "CFX-A01",
                "results": [
                    {"code": "File Name", "value": "CFX Sansure"},
                    {"code": "Notes", "value": "Plaque1"},
                    {
                        "code": "Created By User",
                        "value": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {"code": "RACK", "value": "A01"},
                    {
                        "code": "IPC",
                        "instrument": "SIM01944",
                        "value": "14.76039106",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {
                        "code": "Cy5IPC",
                        "instrument": "SIM01944",
                        "value": "14.76039106",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                ],
                "sample_id": "123456",
                "type": "patient",
            },
            {
                "rack": "CFX-A02",
                "results": [
                    {"code": "File Name", "value": "CFX Sansure"},
                    {"code": "Notes", "value": "Plaque1"},
                    {
                        "code": "Created By User",
                        "value": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {"code": "RACK", "value": "A02"},
                    {
                        "code": "IPC",
                        "instrument": "SIM01944",
                        "value": "12.38063612",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {
                        "code": "Cy5IPC",
                        "instrument": "SIM01944",
                        "value": "12.38063612",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                ],
                "sample_id": "135790",
                "type": "patient",
            },
            {
                "rack": "CFX-A03",
                "results": [
                    {"code": "File Name", "value": "CFX Sansure"},
                    {"code": "Notes", "value": "Plaque1"},
                    {
                        "code": "Created By User",
                        "value": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {"code": "RACK", "value": "A03"},
                    {
                        "code": "IPC",
                        "instrument": "SIM01944",
                        "value": "12.37989111",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {
                        "code": "Cy5IPC",
                        "instrument": "SIM01944",
                        "value": "12.37989111",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                ],
                "sample_id": "987652",
                "type": "patient",
            },
            {
                "rack": "CFX-A04",
                "results": [
                    {"code": "File Name", "value": "CFX Sansure"},
                    {"code": "Notes", "value": "Plaque1"},
                    {
                        "code": "Created By User",
                        "value": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {"code": "RACK", "value": "A04"},
                    {
                        "code": "IPC",
                        "instrument": "SIM01944",
                        "value": "12.81727096",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                    {
                        "code": "Cy5IPC",
                        "instrument": "SIM01944",
                        "value": "12.81727096",
                        "date": date_resu,
                        "comments": ["Plaque1"],
                        "operator": "adminavec des accent èà ^ðŋħĸ",
                    },
                ],
                "sample_id": "123453",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected
    self.configure(analyseur_traites="SIM01944")
    worklist = self.decode(trames)
    assert worklist == expected
    self.configure(analyseur_traites="")
    worklist = self.decode(trames)
    assert worklist == expected
    self.configure(analyseur_traites="XXXX")
    worklist = self.decode(trames)
    assert len(worklist.samples) == 0


def test_decode_extra_results(self):
    self.configure(extra_results=True)
    worklist = self.decode(
        """
        File Name,testvivien2.pcrd,,,,,,
    Created By User,admin,,,,,,
    Notes,notes,,,,,,
    ID,,,,,,,
    Run Started,05/31/2021 10:49:48 UTC,,,,,,
    Run Ended,05/31/2021 12:01:30 UTC,,,,,,
    Sample Vol,13,,,,,,
    Lid Temp,105,,,,,,
    Protocol File Name,Protocole kit QUADRIPLEX IdSol.prcl,,,,,,
    Plate Setup File Name,CRIB-2021.05.31.S1.ECFX3.AMGIA.pltd,,,,,,
    Base Serial Number,CT047811,,,,,,
    Optical Head Serial Number,785BR25721,,,,,,
    CFX Maestro Version,4.1.2433.1219. ,,,,,,
    ,,,,,,,
    Well group,All Wells,,,,,,
    Amplification step,4,,,,,,
    Melt step,,,,,,,
    ,,,,,,,
    ,,,,,,,
    Well,Fluor,Target,Content,Sample,Cq,Call,End RFU
    C06,Cy5,Endogene,Unkn-30,19E180031548,25.0059924876803,,2107.34494280569
    C06,FAM,SARS-CoV-2,Unkn-30,19E180031548,NaN,,237.933427943401
    C06,HEX,SPIKE N501Y,Unkn-30,19E180031548,NaN,,6.19242099372141
    C06,ROX,SPIKE E484K,Unkn-30,19E180031548,NaN,,10.0838490882084
    """
    )
    date_resu = datetime.datetime(2021, 5, 31, 12, 1, 30)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "CFX-C06",
                "sample_id": "19E180031548",
                "type": "patient",
                "results": [
                    {"code": "File Name", "value": "testvivien2"},
                    {"code": "Notes", "value": "notes"},
                    {"code": "Created By User", "value": "admin"},
                    {"code": "RACK", "value": "C06"},
                    {
                        "code": "Cy5EndogeneCq",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "25.0059924876803",
                    },
                    {
                        "code": "Cy5EndogeneEnd RFU",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "2107.34494280569",
                    },
                    {
                        "code": "FAMSARS-CoV-2Cq",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "NaN",
                    },
                    {
                        "code": "FAMSARS-CoV-2End RFU",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "237.933427943401",
                    },
                    {
                        "code": "HEXSPIKE N501YCq",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "NaN",
                    },
                    {
                        "code": "HEXSPIKE N501YEnd RFU",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "6.19242099372141",
                    },
                    {
                        "code": "ROXSPIKE E484KCq",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "NaN",
                    },
                    {
                        "code": "ROXSPIKE E484KEnd RFU",
                        "comments": ["notes"],
                        "date": date_resu,
                        "instrument": "CT047811",
                        "operator": "admin",
                        "value": "10.0838490882084",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_encode_order(self):
    worklist = driverng.Worklist(
        context="order",
        origin="plandeplaque",
        samples=[
            {
                "sample_id": "123456",
                "type": "patient",
                "geometry": {
                    "format_abscissa": "n1",
                    "format_ordinate": "al",
                    "plate_size": "96",
                    "height": 12,
                    "width": 8,
                    "run_protocol": "Protocole kit Variants",
                },
                "wells": [
                    {
                        "abscissa": "1",
                        "barcode": "19E340041469",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E320083243",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E111106848",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E320083201",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E290222794",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E210400490",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E120261189",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E340041496",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E111106830",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E210400673",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E290222796",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E250183743",
                        "ordinate": "A",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E290222772",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E240232355",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E320083236",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E111106608",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E240232339",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E290222811",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E310091221",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E320083207",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E111105914",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E120261214",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E130597747",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E111106837",
                        "ordinate": "B",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E320083237",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E320083223",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E111106330",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E290222759",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E320083245",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E310091216",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E320083206",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E290222813",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E320083227",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E120261203",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E111106697",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E290222791",
                        "ordinate": "C",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E290222790",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E240232345",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E320083233",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E320083199",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E310091160",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E310091191",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E290222786",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E310091181",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E111106475",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E111106792",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E320083252",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E111106695",
                        "ordinate": "D",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E111106623",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E154405251",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E130597791",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E130597804",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E111106454",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E310091220",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E320083217",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E130597757",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E330098936",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E290222828",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E270247782",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E340041508",
                        "ordinate": "E",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E111106274",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E130597755",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E111106391",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E290222858",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E111106596",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E130597808",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E111106112",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E320083238",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E111106344",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E340041561",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E310091196",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E290222775",
                        "ordinate": "F",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E111106614",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E111106553",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E111106364",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E320083228",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E130597797",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E111106541",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E240232357",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E290222821",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E210400628",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E340041541",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "19E111106603",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "19E290222750",
                        "ordinate": "G",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "1",
                        "barcode": "19E111106507",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "2",
                        "barcode": "19E340041554",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "3",
                        "barcode": "19E340041581",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "4",
                        "barcode": "19E111106404",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "5",
                        "barcode": "19E340041465",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "6",
                        "barcode": "19E111106342",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "7",
                        "barcode": "19E340041529",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "8",
                        "barcode": "19E290222835",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "9",
                        "barcode": "19E130597777",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "10",
                        "barcode": "19E240232356",
                        "ordinate": "H",
                        "well_type": 0,
                    },
                    {
                        "abscissa": "11",
                        "barcode": "Negative Control",
                        "ordinate": "H",
                        "well_type": 1,
                    },
                    {
                        "abscissa": "12",
                        "barcode": "Positive Control",
                        "ordinate": "H",
                        "well_type": 1,
                    },
                ],
                "orders": [
                    {
                        "code": "SARS-CoV-2;Spike N501Y;Spike E484K;Contrôle endogene",
                        "action": "new",
                        "priority": "A",
                    }
                ],
            }
        ],
    )
    message = self.order(worklist=worklist)
    expected = '''
    Plate Header,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Filed,Data,,Instruction,,,,,,,,,,,,,,,,,,,,,,,,,,
    Version,1,,Do not modify this field.,,,,,,,,,,,,,,,,,,,,,,,,,,
    Plate Size,96,,Do not modify this field.,,,,,,,,,,,,,,,,,,,,,,,,,,
    Plate Type,BR Clear,,Allowed values (BR White,BR Clear),,,,,,,,,,,,,,,,,,,,,,,,,,
    Scan Mode,All Channels,,"Allowed values (""SYBR/FAM Only"""," ""All Channels"""," ""FRET"")",,,,,,,,,,,,,,,,,,,,,,,,
    Units,copy number,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Run ID,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Run Notes,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Run Protocol,Protocole kit Variants.prcl,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Data File,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    TBD,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Plate Data,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
    Well,Ch1 Dye,Ch2 Dye,Ch3 Dye,Ch4 Dye,Ch5 Dye,FRET,Sample Type,Sample Name,Ch1 Target Name,Ch2 Target Name,Ch3 Target Name,Ch4 Target Name,Ch5 Target Name,FRET Target Name,Biological Set Name,Replicate,Ch1 Quantity,Ch2 Quantity,Ch3 Quantity,Ch4 Quantity,Ch5 Quantity,FRET Quantity,Well Note,Ch1 Well Color,Ch2 Well Color,Ch3 Well Color,Ch4 Well Color,Ch5 Well Color,FRET Well Color
    A1,FAM,HEX,Texas Red,Cy5,,,0,19E340041469,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,1,,,,,,,,,,,,,
    A2,FAM,HEX,Texas Red,Cy5,,,0,19E320083243,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,2,,,,,,,,,,,,,
    A3,FAM,HEX,Texas Red,Cy5,,,0,19E111106848,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,3,,,,,,,,,,,,,
    A4,FAM,HEX,Texas Red,Cy5,,,0,19E320083201,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,4,,,,,,,,,,,,,
    A5,FAM,HEX,Texas Red,Cy5,,,0,19E290222794,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,5,,,,,,,,,,,,,
    A6,FAM,HEX,Texas Red,Cy5,,,0,19E210400490,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,6,,,,,,,,,,,,,
    A7,FAM,HEX,Texas Red,Cy5,,,0,19E120261189,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,7,,,,,,,,,,,,,
    A8,FAM,HEX,Texas Red,Cy5,,,0,19E340041496,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,8,,,,,,,,,,,,,
    A9,FAM,HEX,Texas Red,Cy5,,,0,19E111106830,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,9,,,,,,,,,,,,,
    A10,FAM,HEX,Texas Red,Cy5,,,0,19E210400673,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,10,,,,,,,,,,,,,
    A11,FAM,HEX,Texas Red,Cy5,,,0,19E290222796,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,11,,,,,,,,,,,,,
    A12,FAM,HEX,Texas Red,Cy5,,,0,19E250183743,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,12,,,,,,,,,,,,,
    B1,FAM,HEX,Texas Red,Cy5,,,0,19E290222772,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,13,,,,,,,,,,,,,
    B2,FAM,HEX,Texas Red,Cy5,,,0,19E240232355,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,14,,,,,,,,,,,,,
    B3,FAM,HEX,Texas Red,Cy5,,,0,19E320083236,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,15,,,,,,,,,,,,,
    B4,FAM,HEX,Texas Red,Cy5,,,0,19E111106608,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,16,,,,,,,,,,,,,
    B5,FAM,HEX,Texas Red,Cy5,,,0,19E240232339,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,17,,,,,,,,,,,,,
    B6,FAM,HEX,Texas Red,Cy5,,,0,19E290222811,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,18,,,,,,,,,,,,,
    B7,FAM,HEX,Texas Red,Cy5,,,0,19E310091221,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,19,,,,,,,,,,,,,
    B8,FAM,HEX,Texas Red,Cy5,,,0,19E320083207,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,20,,,,,,,,,,,,,
    B9,FAM,HEX,Texas Red,Cy5,,,0,19E111105914,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,21,,,,,,,,,,,,,
    B10,FAM,HEX,Texas Red,Cy5,,,0,19E120261214,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,22,,,,,,,,,,,,,
    B11,FAM,HEX,Texas Red,Cy5,,,0,19E130597747,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,23,,,,,,,,,,,,,
    B12,FAM,HEX,Texas Red,Cy5,,,0,19E111106837,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,24,,,,,,,,,,,,,
    C1,FAM,HEX,Texas Red,Cy5,,,0,19E320083237,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,25,,,,,,,,,,,,,
    C2,FAM,HEX,Texas Red,Cy5,,,0,19E320083223,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,26,,,,,,,,,,,,,
    C3,FAM,HEX,Texas Red,Cy5,,,0,19E111106330,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,27,,,,,,,,,,,,,
    C4,FAM,HEX,Texas Red,Cy5,,,0,19E290222759,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,28,,,,,,,,,,,,,
    C5,FAM,HEX,Texas Red,Cy5,,,0,19E320083245,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,29,,,,,,,,,,,,,
    C6,FAM,HEX,Texas Red,Cy5,,,0,19E310091216,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,30,,,,,,,,,,,,,
    C7,FAM,HEX,Texas Red,Cy5,,,0,19E320083206,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,31,,,,,,,,,,,,,
    C8,FAM,HEX,Texas Red,Cy5,,,0,19E290222813,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,32,,,,,,,,,,,,,
    C9,FAM,HEX,Texas Red,Cy5,,,0,19E320083227,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,33,,,,,,,,,,,,,
    C10,FAM,HEX,Texas Red,Cy5,,,0,19E120261203,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,34,,,,,,,,,,,,,
    C11,FAM,HEX,Texas Red,Cy5,,,0,19E111106697,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,35,,,,,,,,,,,,,
    C12,FAM,HEX,Texas Red,Cy5,,,0,19E290222791,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,36,,,,,,,,,,,,,
    D1,FAM,HEX,Texas Red,Cy5,,,0,19E290222790,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,37,,,,,,,,,,,,,
    D2,FAM,HEX,Texas Red,Cy5,,,0,19E240232345,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,38,,,,,,,,,,,,,
    D3,FAM,HEX,Texas Red,Cy5,,,0,19E320083233,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,39,,,,,,,,,,,,,
    D4,FAM,HEX,Texas Red,Cy5,,,0,19E320083199,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,40,,,,,,,,,,,,,
    D5,FAM,HEX,Texas Red,Cy5,,,0,19E310091160,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,41,,,,,,,,,,,,,
    D6,FAM,HEX,Texas Red,Cy5,,,0,19E310091191,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,42,,,,,,,,,,,,,
    D7,FAM,HEX,Texas Red,Cy5,,,0,19E290222786,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,43,,,,,,,,,,,,,
    D8,FAM,HEX,Texas Red,Cy5,,,0,19E310091181,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,44,,,,,,,,,,,,,
    D9,FAM,HEX,Texas Red,Cy5,,,0,19E111106475,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,45,,,,,,,,,,,,,
    D10,FAM,HEX,Texas Red,Cy5,,,0,19E111106792,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,46,,,,,,,,,,,,,
    D11,FAM,HEX,Texas Red,Cy5,,,0,19E320083252,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,47,,,,,,,,,,,,,
    D12,FAM,HEX,Texas Red,Cy5,,,0,19E111106695,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,48,,,,,,,,,,,,,
    E1,FAM,HEX,Texas Red,Cy5,,,0,19E111106623,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,49,,,,,,,,,,,,,
    E2,FAM,HEX,Texas Red,Cy5,,,0,19E154405251,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,50,,,,,,,,,,,,,
    E3,FAM,HEX,Texas Red,Cy5,,,0,19E130597791,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,51,,,,,,,,,,,,,
    E4,FAM,HEX,Texas Red,Cy5,,,0,19E130597804,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,52,,,,,,,,,,,,,
    E5,FAM,HEX,Texas Red,Cy5,,,0,19E111106454,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,53,,,,,,,,,,,,,
    E6,FAM,HEX,Texas Red,Cy5,,,0,19E310091220,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,54,,,,,,,,,,,,,
    E7,FAM,HEX,Texas Red,Cy5,,,0,19E320083217,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,55,,,,,,,,,,,,,
    E8,FAM,HEX,Texas Red,Cy5,,,0,19E130597757,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,56,,,,,,,,,,,,,
    E9,FAM,HEX,Texas Red,Cy5,,,0,19E330098936,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,57,,,,,,,,,,,,,
    E10,FAM,HEX,Texas Red,Cy5,,,0,19E290222828,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,58,,,,,,,,,,,,,
    E11,FAM,HEX,Texas Red,Cy5,,,0,19E270247782,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,59,,,,,,,,,,,,,
    E12,FAM,HEX,Texas Red,Cy5,,,0,19E340041508,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,60,,,,,,,,,,,,,
    F1,FAM,HEX,Texas Red,Cy5,,,0,19E111106274,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,61,,,,,,,,,,,,,
    F2,FAM,HEX,Texas Red,Cy5,,,0,19E130597755,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,62,,,,,,,,,,,,,
    F3,FAM,HEX,Texas Red,Cy5,,,0,19E111106391,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,63,,,,,,,,,,,,,
    F4,FAM,HEX,Texas Red,Cy5,,,0,19E290222858,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,64,,,,,,,,,,,,,
    F5,FAM,HEX,Texas Red,Cy5,,,0,19E111106596,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,65,,,,,,,,,,,,,
    F6,FAM,HEX,Texas Red,Cy5,,,0,19E130597808,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,66,,,,,,,,,,,,,
    F7,FAM,HEX,Texas Red,Cy5,,,0,19E111106112,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,67,,,,,,,,,,,,,
    F8,FAM,HEX,Texas Red,Cy5,,,0,19E320083238,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,68,,,,,,,,,,,,,
    F9,FAM,HEX,Texas Red,Cy5,,,0,19E111106344,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,69,,,,,,,,,,,,,
    F10,FAM,HEX,Texas Red,Cy5,,,0,19E340041561,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,70,,,,,,,,,,,,,
    F11,FAM,HEX,Texas Red,Cy5,,,0,19E310091196,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,71,,,,,,,,,,,,,
    F12,FAM,HEX,Texas Red,Cy5,,,0,19E290222775,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,72,,,,,,,,,,,,,
    G1,FAM,HEX,Texas Red,Cy5,,,0,19E111106614,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,73,,,,,,,,,,,,,
    G2,FAM,HEX,Texas Red,Cy5,,,0,19E111106553,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,74,,,,,,,,,,,,,
    G3,FAM,HEX,Texas Red,Cy5,,,0,19E111106364,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,75,,,,,,,,,,,,,
    G4,FAM,HEX,Texas Red,Cy5,,,0,19E320083228,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,76,,,,,,,,,,,,,
    G5,FAM,HEX,Texas Red,Cy5,,,0,19E130597797,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,77,,,,,,,,,,,,,
    G6,FAM,HEX,Texas Red,Cy5,,,0,19E111106541,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,78,,,,,,,,,,,,,
    G7,FAM,HEX,Texas Red,Cy5,,,0,19E240232357,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,79,,,,,,,,,,,,,
    G8,FAM,HEX,Texas Red,Cy5,,,0,19E290222821,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,80,,,,,,,,,,,,,
    G9,FAM,HEX,Texas Red,Cy5,,,0,19E210400628,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,81,,,,,,,,,,,,,
    G10,FAM,HEX,Texas Red,Cy5,,,0,19E340041541,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,82,,,,,,,,,,,,,
    G11,FAM,HEX,Texas Red,Cy5,,,0,19E111106603,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,83,,,,,,,,,,,,,
    G12,FAM,HEX,Texas Red,Cy5,,,0,19E290222750,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,84,,,,,,,,,,,,,
    H1,FAM,HEX,Texas Red,Cy5,,,0,19E111106507,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,85,,,,,,,,,,,,,
    H2,FAM,HEX,Texas Red,Cy5,,,0,19E340041554,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,86,,,,,,,,,,,,,
    H3,FAM,HEX,Texas Red,Cy5,,,0,19E340041581,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,87,,,,,,,,,,,,,
    H4,FAM,HEX,Texas Red,Cy5,,,0,19E111106404,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,88,,,,,,,,,,,,,
    H5,FAM,HEX,Texas Red,Cy5,,,0,19E340041465,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,89,,,,,,,,,,,,,
    H6,FAM,HEX,Texas Red,Cy5,,,0,19E111106342,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,90,,,,,,,,,,,,,
    H7,FAM,HEX,Texas Red,Cy5,,,0,19E340041529,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,91,,,,,,,,,,,,,
    H8,FAM,HEX,Texas Red,Cy5,,,0,19E290222835,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,92,,,,,,,,,,,,,
    H9,FAM,HEX,Texas Red,Cy5,,,0,19E130597777,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,93,,,,,,,,,,,,,
    H10,FAM,HEX,Texas Red,Cy5,,,0,19E240232356,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,94,,,,,,,,,,,,,
    H11,FAM,HEX,Texas Red,Cy5,,,1,Negative Control,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,1,,,,,,,,,,,,,
    H12,FAM,HEX,Texas Red,Cy5,,,1,Positive Control,SARS-CoV-2,Spike N501Y,Spike E484K,Contrôle endogene,,,,1,,,,,,,,,,,,,
    '''
    assert message == expected
