# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime


parser_class = "dm"


def test_order(self):
    result = self.order()
    expected = """H|\\^&||||||||||P|1394-97|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||||||||||||||LOC
        C|1|P|Patient commentaire no1|G
        C|2|P|Patient commentaire no2|G
        C|3|P|Commentaire patient no2|G
        L|1"""
    assert result == expected

    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].results.extend(
        [{"code": "WBC", "value": "42.42"}, {"code": "RBC", "value": "24.12"}]
    )
    worklist.samples[0].orders.extend(
        [
            {"code": "WBC", "action": "add", "priority": "A"},
            {"code": "PLT", "action": "add", "priority": "A"},
            {"code": "RBC", "action": "add", "priority": "A"},
        ]
    )
    message = self.order(worklist=worklist)
    expected = """H|\\^&||||||||||P|1394-97|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||||||||||||||LOC
        C|1|P|Patient commentaire no1|G
        C|2|P|Patient commentaire no2|G
        C|3|P|Commentaire patient no2|G
        O|1|123456||^WBC^^\\^RBC^^\\^PLT^^|S||20130513114242||||A||Information clinique|||PRESCRIBER||||100^\\^\\^|42.42^24.1||||Q
        C|1|P|Sample commentaire|G
        L|1
    """
    assert message == expected

    # Nb cellule paramétré
    self.configure(nombre_cellules="120")
    message = self.order(worklist=worklist)
    expected = """H|\\^&||||||||||P|1394-97|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||||||||||||||LOC
        C|1|P|Patient commentaire no1|G
        C|2|P|Patient commentaire no2|G
        C|3|P|Commentaire patient no2|G
        O|1|123456||^WBC^^\\^RBC^^\\^PLT^^|S||20130513114242||||A||Information clinique|||PRESCRIBER||||120^\\^\\^|42.42^24.1||||Q
        C|1|P|Sample commentaire|G
        L|1"""
    assert message == expected

    # Nb cellule dynamique
    worklist.samples[0].results.extend([{"code": "NB_CELLULES", "value": "110"}])
    message = self.order(worklist=worklist)
    expected = """H|\\^&||||||||||P|1394-97|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||||||||||||||LOC
        C|1|P|Patient commentaire no1|G
        C|2|P|Patient commentaire no2|G
        C|3|P|Commentaire patient no2|G
        O|1|123456||^WBC^^\\^RBC^^\\^PLT^^|S||20130513114242||||A||Information clinique|||PRESCRIBER||||110^\\^\\^|42.42^24.1||||Q
        C|1|P|Sample commentaire|G
        L|1"""
    assert message == expected


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||CellaVision|CellaLabs||555-WRONG||LIS-System||P|1394-97|20210701114024
    Q|1|^4082801301^||ALL
    L|1|
    """
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "4082801301", "type": "patient"}],
    }
    assert result == expected


def test_result(self):
    date = datetime.datetime(2013, 10, 30, 11, 16, 19)
    result = self.decode(
        """
        H|\\^&|||CellaVision|CellaLabs||555-WRONG||LIS-System||P|1394-97|20131030111620
    P|1
    O|1|30307084||^WBC^^\\^RBC^^|R||20131030105551||||A|||||||||120^\\^|0.00^0.0|||33215|F
    C|1|P|Remarque reçue: &XD&&XA&Commentaire recu&XD&&XA&Remarques de l'utilisateur: &XD&&XA&Commentaire utilisateur&XD&&XA&Remarques sur la lame: &XD&&XA&Lame&XD&&XA&|G
    R|3|^WBC^^EO/#|1||0 to 114|N||F||^CHRISTINE||20131030111619
    R|4|^WBC^^EO/%|0.9|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|5|^WBC^^BA/#|2||0 to 114|N||F||^CHRISTINE||20131030111619
    R|6|^WBC^^BA/%|1.7|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|7|^WBC^^LY/#|30||0 to 114|N||F||^CHRISTINE||20131030111619
    R|8|^WBC^^LY/%|26.3|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|9|^WBC^^MO/#|5||0 to 114|N||F||^CHRISTINE||20131030111619
    R|10|^WBC^^MO/%|4.4|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|11|^WBC^^PMY/#|3||0 to 114|N||F||^CHRISTINE||20131030111619
    R|12|^WBC^^PMY/%|2.6|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|13|^WBC^^MY/#|4||0 to 114|N||F||^CHRISTINE||20131030111619
    R|14|^WBC^^MY/%|3.5|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|15|^WBC^^MMY/#|1||0 to 114|N||F||^CHRISTINE||20131030111619
    R|16|^WBC^^MMY/%|0.9|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|17|^WBC^^BL/#|1||0 to 114|N||F||^CHRISTINE||20131030111619
    R|18|^WBC^^BL/%|0.9|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|19|^WBC^^PC/#|2||0 to 114|N||F||^CHRISTINE||20131030111619
    R|20|^WBC^^PC/%|1.8|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|21|^WBC^^SMU/#|18||0 to 114|N||F||^CHRISTINE||20131030111619
    R|22|^WBC^^SMU/%|15.8|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|23|^WBC^^ERB/#|2||0 to 114|N||F||^CHRISTINE||20131030111619
    R|24|^WBC^^ERB/%|1.8|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|25|^WBC^^ART/#|80||0 to 114|N||F||^CHRISTINE||20131030111619
    R|26|^WBC^^ART/%|70.2|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|27|^WBC^^GT/#|6||0 to 114|N||F||^CHRISTINE||20131030111619
    R|28|^WBC^^GT/%|5.3|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|29|^WBC^^TAG/#|81||0 to 114|N||F||^CHRISTINE||20131030111619
    R|30|^WBC^^TAG/%|71.1|%|0 to 100|N||F||^CHRISTINE||20131030111619
    R|31|^WBC^^USRWBC1/#|1||0 to 114|N||F||^CHRISTINE||20131030111619
    R|32|^WBC^^USRWBC1/%|0.9|%|0 to 100|N||F||^CHRISTINE||20131030111619
    L|1|"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "30307084",
                "type": "patient",
                "results": [
                    {
                        "date": date,
                        "value": "1",
                        "code": "EO/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "0.9",
                        "code": "EO/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "2",
                        "code": "BA/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "1.7",
                        "code": "BA/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "30",
                        "code": "LY/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "26.3",
                        "code": "LY/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "5",
                        "code": "MO/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "4.4",
                        "code": "MO/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "3",
                        "code": "PMY/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "2.6",
                        "code": "PMY/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "4",
                        "code": "MY/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "3.5",
                        "code": "MY/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "1",
                        "code": "MMY/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "0.9",
                        "code": "MMY/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "1",
                        "code": "BL/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "0.9",
                        "code": "BL/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "2",
                        "code": "PC/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "1.8",
                        "code": "PC/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "18",
                        "code": "SMU/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "15.8",
                        "code": "SMU/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "2",
                        "code": "ERB/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "1.8",
                        "code": "ERB/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "80",
                        "code": "ART/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "70.2",
                        "code": "ART/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "6",
                        "code": "GT/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "5.3",
                        "code": "GT/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "81",
                        "code": "TAG/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "71.1",
                        "code": "TAG/%",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "1",
                        "code": "USRWBC1/#",
                        "operator": "CHRISTINE",
                    },
                    {
                        "date": date,
                        "value": "0.9",
                        "code": "USRWBC1/%",
                        "operator": "CHRISTINE",
                    },
                    {"date": date, "value": "Commentaire recu", "code": "COM_SIL"},
                    {"date": date, "value": "Commentaire recu", "code": "COM_SIL1"},
                    {
                        "date": date,
                        "value": "Commentaire utilisateur",
                        "code": "COM_USER",
                    },
                    {
                        "date": date,
                        "value": "Commentaire utilisateur",
                        "code": "COM_USER1",
                    },
                    {"date": date, "value": "Lame", "code": "COM_SLIDE"},
                    {"date": date, "value": "Lame", "code": "COM_SLIDE1"},
                    {"date": None, "value": "0", "code": "SNE/%"},
                ],
            }
        ],
    }
    assert result == expected


def test_no_result(self):
    self.configure(dm_wbcconfirm=True)
    result = self.decode(
        """
    H|\\^&|||CellaVision|CellaLabs||555-WRONG||LIS-System||P|1394-97|20210705141241
    P|1
    O|1|1086105101||^RBC^^|R||20210705094800||||A|||||BAYMC||||^|0.00^0.0^14.0^43.8^80.7^25.8^31.9^0.0^^^^^^64.3^23.9^9.7^1.5^0.6^0.0^0.0^^^^^|||37055|F
    C|1|P|Remarque reçue: &XD&&XA&&XD&&XA&Remarques de l'utilisateur: &XD&&XA&&XD&&XA&Remarques sur la lame: &XD&&XA&&XD&&XA&|G
    L|1|
    """
    )
    expected = {
        "context": "result",
        "samples": [{"results": [], "sample_id": "1086105101", "type": "patient"}],
    }
    assert result == expected
