# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "piano_sil"


def test_decode_order(self):
    worklist = self.decode(
        """
    H|\\^&|||LIS
    P|1|96001|||TEST^ANONYME||19601002|M
    O|1|96000100000001||^^^001^51\\^^^002\\^^^003^100|||||||||||Sp.1
    L|1
    """
    )
    expected = {
        "context": "order",
        "sender_id": "LIS",
        "receiver_id": "HBDRIVE",
        "samples": [
            {
                "type": "patient",
                "sample_id": "96000100000001",
                "request_id": "96000100000001",
                "birthdate": datetime.datetime(1960, 10, 2, 0, 0),
                "first_name": "ANONYME",
                "last_name": "TEST",
                "orders": [
                    {"code": "001", "action": "add", "dilution": "51"},
                    {"code": "002", "action": "add", "dilution": ""},
                    {"code": "003", "action": "add", "dilution": "100"},
                ],
                "patient_id": "96001",
                "sex": "M",
                "specimen": "Sp.1",
                "priority": "",
            }
        ],
    }
    assert worklist == expected


def test_encode_result_piano(self):
    result_worklist = copy.deepcopy(self.result_worklist)
    result_worklist.samples[0].specimen = "Sp.1"
    codes = ["551", "552", "553"]
    dilution = ["100", "", "50"]
    for i, result in enumerate(result_worklist.samples[0].results):
        result.code = codes[i]
        result.dilution = dilution[i]
        result.operator = "VALIDEUR"
    message = self.result(worklist=result_worklist)
    expected = """
    H|\\^&|||INST|||||||||20130715141912
    P|1|L036666666|||NYME^ANO||20110513121500|F
    O|1|123456||^^^551^1|||||||||||Sp.1
    R|1|^^^551|42|mV||N||F||VAL||20110506121400
    O|2|123456||^^^552^1|||||||||||Sp.1
    R|1|^^^552|NORESULT|mV||LL||F||VAL||20110506121400
    O|3|123456||^^^553^1|||||||||||Sp.1
    R|1|^^^553|42|mV||N||F||VAL||20110506121400
    L|1
    """
    assert message == expected


def test_encode_result_with_piano_etendu(self):
    self.configure(astm_piano_etendu=True)
    result_worklist = copy.deepcopy(self.result_worklist)
    codes = ["551", "552", "553"]
    dilution = ["100", "", "50"]
    for i, result in enumerate(result_worklist.samples[0].results):
        result.code = codes[i]
        result.dilution = dilution[i]
        result.operator = "VALIDEUR"
    message = self.result(worklist=result_worklist)
    expected = """
    H|\\^&|||INST|||||||||20130715141912
    P|1|L036666666|||NYME^ANO||20110513121500|F
    O|1|123456||^^^551^1|||||||N||||Sp.1
    R|1|^^^551|42|mV||N||F||VAL||20110506121400|INST
    O|2|123456||^^^552^1|||||||N||||Sp.1
    R|1|^^^552|NORESULT|mV||LL||F||VAL||20110506121400|INST
    O|3|123456||^^^553^1|||||||N||||Sp.1
    R|1|^^^553|42|mV||N||F||VAL||20110506121400|INST
    L|1
    """
    assert message == expected


def test_encode_control_with_piano_etendu(self):
    self.configure(astm_piano_etendu=True)
    result_control_worklist = copy.deepcopy(self.result_control_worklist)
    message = self.result_control(worklist=result_control_worklist)
    expected = """
    H|\\^&|||INST|||||||||20130715141912
    P|1
    O|1|CQ123456||^^^A^1|||||||Q||||Sp.1
    R|1|^^^A|42|mV||HH||F||auto||20110506121400|INST
    L|1
    H|\\^&|||INST|||||||||20130715141912
    P|1
    O|1|CQ123456||^^^B^1|||||||Q||||Sp.1
    R|1|^^^B|4.40|||N||F||auto||20110506121300|INST
    L|1
    H|\\^&|||INST|||||||||20130715141912
    P|1
    O|1|CQ123488||^^^A^1|||||||Q||||Sp.1
    R|1|^^^A|4|mV||N||F||auto||20110506121400|INST
    L|1
    """
    assert message == expected
