# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "poccelerator"


def test_encode_order(self):
    self.configure(nom_recepteur="FACILITY")
    message = self.order()
    expected = """
    MSH|^~\\&|MCA|MCA|POCS|FACILITY|20130715141912||ADT^A04||P|2.4|||AL|NE||8859/1
    PID|1|036666666|L036666666||NYME^ANO||20110513|F|||45 rue lambda\\S\\lala^^Toulouse^^31000^FR
    PV1|1|O|LOC||||||ATTENDING||||||||||123456||||||||||||||||||||||||||20130511103030
    """
    assert message == expected


def test_encode_query(self):
    message = self.order(
        context={
            "query": self.parse(
                """
            MSH|^~\\&|Meditrac||||20120716140232||QRY^A19|206373478|P|2.4|||AL|NE||8859/1
            QRD|20120716140232|R|I|Q206373478|||1|1000001652|DEM|ALL
            QRF|||
            """
            )
        }
    )
    expected = """
    MSH|^~\\&|MCA|MCA|POCS|GHRMSA|20130715141912||ADR^A19|206373478|P|2.4|||AL|NE||8859/1
    MSA|AA|206373478|||F
    QRD|20120716140232|R|I|Q206373478|||1|1000001652|DEM|ALL
    PID|1|036666666|L036666666||NYME^ANO||20110513|F|||45 rue lambda\\S\\lala^^Toulouse^^31000^FR
    PV1|1|I|LOC||||||ATTENDING||||||||||123456||||||||||||||||||||||||||20130511103030
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Meditrac||||20120716140232||QRY^A19|206373478|P|2.4|||AL|NE||8859/1
    QRD|20120716140232|R|I|Q206373478|||1|1000001652|DEM|ALL
    QRF|||
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "1000001652"}],
    }
    assert worklist == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            MSH|^~\\&|Meditrac||||20120716140232||QRY^A19|206373478|P|2.4|||AL|NE||8859/1
            QRD|20120716140232|R|I|Q206373478|||1|1000001652|DEM|ALL
            QRF|||
            """
            )
        },
    )
    expected = """
    MSH|^~\\&|MCA|MCA|POCS|GHRMSA|20130715141912||ADR^A19|206373478|P|2.4|||AL|NE||8859/1
    MSA|AA|206373478|||F
    QRD|20120716140232|R|I|Q206373478|||1|1000001652|DEM|ALL
    """

    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
        MSH|^~\\&|PCC^GEM 4000;07121016|KH-01|LIS|KH-01|20090924120513||ORU^R01|117725235|P|2.4|||AL|NE||8859/1
        PID|1||2222||Musterfrau^Eva||19550210|F||||||||||||||||||
        PV1|1||32||||Doctor||||||||||||082222|||||||||||||||||||||||||||||||
        ORC|1||||CM||||||||
        OBR|1|||Elektrolyte|||20080225142233|20080225142233|||N|||18991230000000|BLDA||||||||||||^^^20080225142233^^R|||||||
        NTE|1|D|Repeated measurement|
        OBX|1|NM|Ca++^Ca++||1.14|mmol/L|1.17-1.37|N|||F|||20080225142233||||07121016|20080225142233
        OBX|2|NM|Glu^Glu||193|mg/dL|55.0-100.0|N|||F|||20080225142233||||07121016|20080225142233
        OBX|7|NM|pCO2^pCO2||41|mmHg|32.0-46.0||||F|||20080225142233||||07121016|20080225142233
        NTE|1|D|Clot detected|
    """
    )
    date = datetime.datetime(2008, 2, 25, 14, 22, 33)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "type_echantillon", "value": "BLDA"},
                    {
                        "code": "Ca++",
                        "type": "NM",
                        "value": "1.14",
                        "date": date,
                        "flags": ["N"],
                        "unit": "mmol/L",
                        "comments": [],
                        "instrument": "07121016",
                        "operator": "",
                    },
                    {
                        "code": "Glu",
                        "type": "NM",
                        "value": "193",
                        "date": date,
                        "flags": ["N"],
                        "unit": "mg/dL",
                        "comments": [],
                        "instrument": "07121016",
                        "operator": "",
                    },
                    {
                        "code": "pCO2",
                        "type": "NM",
                        "value": "41",
                        "date": date,
                        "flags": [],
                        "unit": "mmHg",
                        "comments": ["Clot detected"],
                        "instrument": "07121016",
                        "operator": "",
                    },
                ],
                "sample_id": "082222",
                "patient_id": "2222",
                "alternate_patient_id": "082222",
                "type": "patient",
                "specimen": "BLDA",
                "first_name": "Eva",
                "last_name": "Musterfrau",
                "sex": "F",
                "birthdate": datetime.datetime(1955, 2, 10, 0, 0),
                "request_date": date,
                "collection_date": datetime.datetime(1899, 12, 30, 0, 0),
                "country": "",
                "request_info": "D\xe9localis\xe9",
                "address": "",
                "postal_code": "",
                "city": "",
                "physicians": [{"code": "Doctor", "type": "attending"}],
                "services": [{"code": "32"}],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(qc_prefix="QC")
    worklist = self.decode(
        """
        MSH|^~\\&|PCC^GEM 4000;07121016|KH-01|LIS|KH-01|20090924120513||ORU^R01|117725235|P|2.4|||AL|NE||8859/1
        PID|1||
        PV1|1||32|
        ORC|1||||CM||||||||
        OBR|1|QC1234||Elektrolyte|||20080225142233|20080225142233|||N|||18991230000000|BLDA||||||||||||^^^20080225142233^^R|||||||
        OBX|1|NM|Ca++^Ca++||1.14|mmol/L|1.17-1.37|N|||F|||20080225142233||||07121016|20080225142233
    """
    )
    date = datetime.datetime(2008, 2, 25, 14, 22, 33)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Ca++",
                        "type": "NM",
                        "value": "1.14",
                        "date": date,
                        "flags": ["N"],
                        "unit": "mmol/L",
                        "comments": [],
                        "instrument": "07121016",
                        "operator": "",
                    }
                ],
                "sample_id": "QC1234",
                "type": "control",
                "specimen": "BLDA",
            }
        ],
    }
    assert worklist == expected
