# -*- coding: utf8 -*-
from __future__ import unicode_literals

parser_class = "suit_uf500"


def test_get_queries(self):
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201608301121
    Q|1||1200937852|||201608301121
    """
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "1200937852", "type": "patient"}],
    }
    assert result == expected


def test_no_order_query_without_option_uf500(self):
    self.configure(uf500_tube_inconnu=False)
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """H|^~\\&|||||||||||A.2|201206251820
        Q|1||1125017208|||201206251820
        L|1||0|3"""
            )
        },
    )
    expected = """H|^~\\&|||||||||||A.2|201307151419
        P|1
        OBR|1|1125017208
        L|1||1|4"""
    assert result == expected


def test_no_order_query_uf500(self):
    self.configure(uf500_tube_inconnu=True)
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """H|^~\\&|||||||||||A.2|201206251820
        Q|1||1125017208|||201206251820
        L|1||0|3"""
            )
        },
    )
    expected = """H|^~\\&|||||||||||A.2|201307151419
        P|1
        OBR|1|1125017208||RBC~XTAL~EC~BACT~WBC
        L|1||1|4"""
    assert result == expected


def test_resultat_uf500(self):
    result = self.decode(
        """H|^~\\&|||||||||||A.2|201608291129
    P|1|||||||U||||||||||||||||||||||||
    OBR|1||1154056852|flag_review~flag_error~flag_iderror~sample_source~sample_color~sample_clarity~vol_sedch~vol_bacch~raw_sedch~raw_bacch~total_sedch~total_bacch~rbc~wbc~ec~cast~bact~path.cast~src~sperm~xtal~ylc~mucus~cond.~rbc-info~cond-info~uti-info~h_rack~h_tube~h_inid~h_inst|||201608291129|||||||201608291129||||||||201608291129||002|||
    OBX|1|NM|FLAG_REVIEW||0||||||F^|||||
    OBX|2|NM|FLAG_ERROR||0||||||F^|||||
    OBX|3|NM|FLAG_IDERROR||0||||||F^|||||
    OBX|4|NM|SAMPLE_SOURCE||0||||||F^|||||
    OBX|5|NM|SAMPLE_COLOR||0||||||F^|||||
    OBX|6|NM|SAMPLE_CLARITY||0||||||F^|||||
    OBX|7|NM|VOL_SEDch||07.80||||||F^|||||
    OBX|8|NM|VOL_BACch||01.00||||||F^|||||
    OBX|9|NM|RAW_SEDch||03859||||||F^|||||
    OBX|10|NM|RAW_BACch||02776||||||F^|||||
    OBX|11|NM|TOTAL_SEDch||004345||||||F^|||||
    OBX|12|NM|TOTAL_BACch||002469||||||F^|||||
    OBX|13|NM|RBC||00015.20|/uL|||||F^|||||
    OBX|14|NM|WBC||00011.40|/uL|||||F^|||||
    OBX|15|NM|EC||00009.30|/uL|||||F^|||||
    OBX|16|NM|CAST||00001.97|/uL|||||F^|||||
    OBX|17|NM|BACT||00012.30|/uL|||||F^|||||
    OBX|18|NM|Path.Cast||00001.44|/uL|||||F^|||||
    OBX|19|NM|SRC||00005.30|/uL|||||F^|||||
    OBX|20|NM|SPERM||00000.00|/uL|||||F^|||||
    OBX|21|NM|XTAL||00000.00|/uL|||||F^|||||
    OBX|22|NM|YLC||00000.00|/uL|||||F^|||||
    OBX|23|NM|MUCUS||00002.10|/uL|||||F^|||||
    OBX|24|NM|COND.||00010.60|mS/cm|||||F^|||||
    OBX|25|NM|RBC-Info||0||||||F^|||||
    OBX|26|NM|Cond-Info||2||||||F^|||||
    OBX|27|NM|UTI-Info||0||||||F^|||||
    OBX|28|NM|H_RACK||2||||||F^|||||
    OBX|29|NM|H_TUBE||8||||||F^|||||
    OBX|30|ST|H_INID||11753||||||F^|||||
    OBX|31|ST|H_INST||UF-500i||||||F^|||||
    L|1||1|35"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "1154056852",
                "rack": "2-008-UF-500i-11753",
                "results": [
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "FLAG_REVIEW",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "FLAG_ERROR",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "FLAG_IDERROR",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "SAMPLE_SOURCE",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "SAMPLE_COLOR",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "SAMPLE_CLARITY",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "07.80",
                        "date": None,
                        "code": "VOL_SEDch",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "01.00",
                        "date": None,
                        "code": "VOL_BACch",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "03859",
                        "date": None,
                        "code": "RAW_SEDch",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "02776",
                        "date": None,
                        "code": "RAW_BACch",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "004345",
                        "date": None,
                        "code": "TOTAL_SEDch",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "002469",
                        "date": None,
                        "code": "TOTAL_BACch",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00015.20",
                        "date": None,
                        "code": "RBC",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00011.40",
                        "date": None,
                        "code": "WBC",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00009.30",
                        "date": None,
                        "code": "EC",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00001.97",
                        "date": None,
                        "code": "CAST",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00012.30",
                        "date": None,
                        "code": "BACT",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00001.44",
                        "date": None,
                        "code": "Path.Cast",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00005.30",
                        "date": None,
                        "code": "SRC",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00000.00",
                        "date": None,
                        "code": "SPERM",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00000.00",
                        "date": None,
                        "code": "XTAL",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00000.00",
                        "date": None,
                        "code": "YLC",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00002.10",
                        "date": None,
                        "code": "MUCUS",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "00010.60",
                        "date": None,
                        "code": "COND.",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "RBC-Info",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "2",
                        "date": None,
                        "code": "Cond-Info",
                        "lots": [],
                    },
                    {
                        "instrument": "UF-500i",
                        "flags": [],
                        "value": "0",
                        "date": None,
                        "code": "UTI-Info",
                        "lots": [],
                    },
                ],
            }
        ],
    }
    assert result == expected
