# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "synergie"


def test_decode_order(self):
    worklist = self.decode(
        """
    H|^~\\&|||LMX5^LABOC||ORM|||38^ONELINK2||P|A.2.|20141125110200|
    P|0001|00555555|0000000123456|0000000123456|TEST^Hemostase1^||19980615|M|||||||||||||||20141014~||LBTP^ROOM^BED^|
    OBR|0001|0195199384^4115199384||RCRD^^L^|R^N|201411251047|201411251047||||R|||201411251047|019^BLEU PALE|||LBTP||Cat5|
    OBX|1|TX|RC^RC^L^||NON RENSEIGNE||||||F|
    OBR|0001|0195199384^4115199384||MF^Monomères de fibrine^L^|R^N|201411251047|201411251047||||R|||201411251047|019^BLEU PALE|||LBTP||Cat5|
    OBX|1|CE|MF^Monomères de fibrine^L^||MF5||||||A|20141124|
    OBX|1|TX|RC^RC^L^||flop||||||F|
    OBX|2|CE|MF^Monomères de fibrine^L^||MF5||||||A|20141124|
    OBR|0001|0195199384^4115199384||CORTE^Cortisol 2^L^|R^N|201411251047|201411251047||||R|||201411251047|019^BLEU PALE|||LBTP||Cat5|
    OBX|1|TX|RC^RC^L^||NON RENSEIGNE||||||F|
    OBX|1|ST|CORTE^Cortisol^L^CORTE^2^L^|CORTE^2|||||||A|
    OBR|0001|0195199384^4115199384||2GB~2GR|R||201411251047||||S|||||||||||||
    OBR|0001|0195199384^4115199384||2TCMH|R||201411251047||||C|||||||||||||
    L|||1|11||"""
    )
    expected = {
        "context": "order",
        "receiver_id": "38",
        "sender_id": "LMX5",
        "samples": [
            {
                "type": "patient",
                "sample_id": "0195199384",
                "admission_date": datetime.datetime(2014, 10, 14, 0, 0),
                "admission_end_date": None,
                "alternate_patient_id": "0000000123456",
                "bed": "BED",
                "birthdate": datetime.datetime(1998, 6, 15, 0, 0),
                "collection_date": datetime.datetime(2014, 11, 25, 10, 47),
                "collection_end_date": None,
                "services": [{"code": "LBTP", "name": ""}],
                "diagnostic": "",
                "first_name": "Hemostase1",
                "height": "",
                "last_name": "TEST",
                "location": "LBTP",
                "maiden_name": "",
                "address": "",
                "country": "",
                "postal_code": "",
                "city": "",
                "physiologie": "",
                "orders": [
                    {"action": "new", "code": "RCRD", "priority": "R"},
                    {"accueil": True, "action": "new", "code": "RC", "priority": "R"},
                    {
                        "action": "new",
                        "code": "MF",
                        "priority": "R",
                        "label": "Monomères de fibrine",
                    },
                    {
                        "action": "new",
                        "code": "CORTE",
                        "priority": "R",
                        "label": "Cortisol 2",
                    },
                    {"action": "del", "code": "2GB", "priority": "R"},
                    {"action": "del", "code": "2GR", "priority": "R"},
                    {"action": "del", "code": "2TCMH", "priority": "R"},
                ],
                "patient_id": "00555555",
                "phone": "",
                "request_date": datetime.datetime(2014, 11, 25, 10, 47),
                "request_id": "4115199384",
                "results": [
                    {
                        "type": "TX",
                        "anterior": False,
                        "code": "RC",
                        "date": None,
                        "sub_code": "",
                        "unit": "",
                        "value": "NON RENSEIGNE;flop",
                    },
                    {
                        "type": "CE",
                        "anterior": True,
                        "code": "MF",
                        "date": datetime.datetime(2014, 11, 24, 0, 0),
                        "sub_code": "",
                        "unit": "",
                        "value": "MF5",
                    },
                ],
                "room": "ROOM",
                "sex": "M",
                "title": "",
                "transcriptionist": "",
                "exe_request_id": "",
                "exe_sample_id": "",
                "birth_row": "",
                "admission_status": "",
                "deathdate": None,
                "nature": {
                    "code": "019",
                    "label": "",
                    "codification": "",
                    "additive": "BLEU PALE",
                },
            }
        ],
    }

    assert worklist == expected


def test_no_ipp(self):
    worklist = self.decode(
        """
    H|^~\\&|||LMX5^LABOC||ORM|||38^ONELINK2||P|A.2.|20150113152415|
    P|1||||LAVIELLE^Lucie||19440502000000|F|||||||||||||||||L95001|
    OBR|1|0128809433^5018809433||CA72P|R||20150105152300||||A||||012|
    L|||1|4"""
    )
    expected = {
        "context": "order",
        "sender_id": "LMX5",
        "receiver_id": "38",
        "samples": [
            {
                "type": "patient",
                "address": "",
                "admission_date": None,
                "admission_end_date": None,
                "alternate_patient_id": "",
                "bed": "",
                "birthdate": datetime.datetime(1944, 5, 2, 0, 0),
                "city": "",
                "collection_date": datetime.datetime(2015, 1, 5, 15, 23),
                "collection_end_date": None,
                "country": "",
                "diagnostic": "",
                "first_name": "Lucie",
                "height": "",
                "last_name": "LAVIELLE",
                "location": "L95001",
                "maiden_name": "",
                "physiologie": "",
                "orders": [{"action": "add", "code": "CA72P", "priority": "R"}],
                "nature": {
                    "code": "012",
                    "label": "",
                    "codification": "",
                    "additive": "",
                },
                "patient_id": "5018809433",
                "phone": "",
                "postal_code": "",
                "request_date": datetime.datetime(2015, 1, 5, 15, 23),
                "request_id": "5018809433",
                "room": "",
                "sample_id": "0128809433",
                "sex": "F",
                "title": "",
                "transcriptionist": "",
                "exe_request_id": "",
                "exe_sample_id": "",
                "birth_row": "",
                "admission_status": "",
                "deathdate": None,
            }
        ],
    }

    assert worklist == expected
